/*
 * smupsd - Monitors an APC Smart-UPS under Linux.
 * Copyright (c) 1996 Netsco, Inc.  All Rights Reserved.
 * Written by David E. Myers <dem@netsco.com>.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * See the file COPYING for details.
 *
 * NETSCO MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
 * OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
 * LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. NETSCO SHALL NOT BE LIABLE
 * FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
 * MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * This program was inspired by these previous works:
 *
 * powerd by Miquel van Smoorenburg and Christian Holtje.
 * apdc   by Pavel Korensky.
 *
 * $Log: UpsMonitor.java,v $
 * Revision 1.6  1997/06/19 20:08:45  dem
 * Added maximum and minimum input voltage indicators and time of last
 * UPS sample based on code from Marc Merlin
 * <marcsoft@magic.metawire.com>.
 *
 * Revision 1.5  1997/03/20 22:46:07  dem
 * Disabled color changes to numeric fields.
 *
 * Revision 1.4  1997/01/27 19:19:12  dem
 * Text field color changing now works.
 *
 * Revision 1.3  1997/01/17 19:04:29  dem
 * Changed the update order.
 *
 * Revision 1.2  1997/01/16 20:39:08  dem
 * Hardcode the name of the GUI file.
 * There must be a better way.
 *
 * Revision 1.1  1997/01/16 20:05:01  dem
 * Initial revision
 *
 *
 */

import java.io.*;
import java.net.*;
import java.awt.*;
import java.util.*;
import marimba.gui.*;


public class UpsMonitor extends PlayerFrame implements Runnable {


    public static void main(String args[]) {
	String host = "localhost";
        if (args.length == 1) {
	    host = args[0]; 
        }
	UpsMonitor um = new UpsMonitor(host);
    }


    public UpsMonitor(String host) {
	this.host = host;
	try {
	    socket = new DatagramSocket();
	}
        catch (SocketException e) {
	    System.err.println(e);
	}
	util.setPresentation("/usr/lib/smupsd/UpsMonitor.gui");
	util.getWidget("upsIdentName").setValue(host);
	show();

	thread = new Thread(this);
	thread.start();
    }


    public void run() {
	while(true) {
	    poll();
	    try {
		thread.sleep(10000);
	    }
	    catch (InterruptedException e) {}
	}
    }


    private void poll() {

	int status = Integer.parseInt(getValue("upsOutputSource"));
	Widget statusBox = this.util.getWidget("upsOutputSource");
	if (status == 5) {
	    statusBox.setForeground(Color.yellow);
	    statusBox.setValue("On Battery");
	}
	else if (status == 3) {
	    statusBox.setForeground(Color.green);
	    statusBox.setValue("On Line");
	}
	else {
	    statusBox.setForeground(Color.red);
	    statusBox.setValue("Unknown");
	}
	    
	updateValue("upsLastSampleDateAndTime");

	updateProgress("upsEstimatedChargeRemaining", 100, 20, false);
	updateProgress("upsOutputPercentLoad", 99, 90, true);

	updateFloatText("upsInputVoltage", 130.0f, 100.0f);
	updateFloatText("upsInputVoltageMin", 130.0f, 100.0f);
	updateFloatText("upsInputVoltageMax", 130.0f, 100.0f);
	updateFloatText("upsOutputVoltage", 130.0f, 100.0f);
	updateFloatText("upsOutputFrequency", 61.0f, 59.0f);
	updateFloatText("upsBatteryVoltage", 30.0f, 20.0f);
	updateFloatText("upsBatteryTemperature", 40.0f, 0.0f);

    }

    private void updateProgress(String parameter, int hi, int lo,
				boolean invert) {
	ProgressIndicatorWidget widget =
	    (ProgressIndicatorWidget)this.util.getWidget(parameter);
	widget.setValue(getValue(parameter));

	if (widget.getPercentage() >= hi) {
	    widget.setBarColor(invert ? Color.red : Color.green);
	}
	else if (widget.getPercentage() <= lo) {
	    widget.setBarColor(invert ? Color.green : Color.red);
	}
	else {
	    widget.setBarColor(Color.yellow);
	}
    }


    private void updateValue(String parameter) {
	Widget widget = this.util.getWidget(parameter);
	String value = getValue(parameter);
	widget.setValue(value);
    }


    private void updateFloatText(String parameter, float hi, float lo) {
	TextBoxWidget widget = (TextBoxWidget)this.util.getWidget(parameter);
	String value = getValue(parameter);
	widget.setValue(value);

	// The existing logic does not take into account
	// 220V feeds and 50Hz line frequencies.
	// if ((Float.valueOf(value).floatValue() < lo) ||
	//     (Float.valueOf(value).floatValue() > hi)) {
	//     widget.setForeground(Color.yellow);
	// }
	// else {
	//     widget.setForeground(Color.green);
	// }
    }


    private String getValue(String parameter) {

	byte buffer[] = new byte[parameter.length()];
	String response = "0";

        try {
	    parameter.getBytes(0, parameter.length(), buffer, 0);
	    DatagramPacket p = new DatagramPacket(buffer, parameter.length(),
			       InetAddress.getByName(host), port);
        
	    socket.send(p);
	    socket.receive(p);

	    response = new String(p.getData(), 0, 0, p.getLength());
        }
        catch (IOException e) {
	    System.err.println(e);
	}

	return response;
    }


    public boolean handleEvent(Event event) {
	if ((event.id == Event.WINDOW_DESTROY)) {
	    System.exit(0);
	}
	else if ((event.id == Event.ACTION_EVENT) && 
	    (event.target instanceof Widget)) {
	    String name = ((Widget)event.target).getName();
	    if (name.equals("quitButton")) {
		System.exit(0);
	    }
	}
	return super.handleEvent(event);
    }

    private Thread thread;
    private DatagramSocket socket;
    private int port = 4321;
    private String host;
}	
